'' =================================================================================================
''
''   File....... jm_cmd_test.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2013 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 15 SEP 2013
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal
' _xinfreq = 6_250_000                                          ' use 6.25MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq               ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con

  RX1 = 31                                                      ' programming / terminal
  TX1 = 30
  
  SDA = 29                                                      ' eeprom / i2c
  SCL = 28


con 

  #(-1), IS_ON, IS_OFF

  #0, LSBFIRST, MSBFIRST
  

con

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR       ' PST formmatting control
  #14, GOTOX, GOTOY, CLS


obj

  term : "fullduplexserial64"
 

var



dat

  ' Format of command string:
  ' ** "P" { position }
  '    -- device #
  '    -- low byte of position
  '    -- high byte of position
  '
  ' ** "W" { wait }
  '    -- low byte of delay (milliseconds)
  '    -- high byte of delay
  '
  ' ** "X" { eXit command sequence }


  CmdTest1              byte    "P", 1,  600 & $FF,  600 >> 8
                        byte    "P", 2, 2400 & $FF, 2400 >> 8
                        byte    "W",    1000 & $FF, 1000 >> 8
                        byte    "P", 1, 2400 & $FF, 2400 >> 8
                        byte    "W",    1500 & $FF, 1500 >> 8    
                        byte    "X"
  

pub main 

  term.start(RX1, TX1, %0000, 115_200)
  pause(10)
  
  term.rxflush { clear rx buffer }
  term.rx      { wait for key }   

  term.tx(CLS)

  run_command_demo(@CmdTest1)

  term.str(string(CR, "Done."))

  repeat
    waitcnt(0)


pub run_command_demo(p_src) | cmd, id, value

'' Extract commands from DAT string at p_src

  repeat
    cmd := byte[p_src++]

    case cmd
      "p", "P":
        id := byte[p_src++]
        value := byte[p_src++]
        value |= byte[p_src++] << 8  
         
        term.str(string("Position", CR))
        term.str(string("-- #"))
        term.dec(id)
        term.tx(CR)
        term.str(string("-- "))
        term.dec(value)
        term.tx(CR) 
         
      "w", "W":
        value := byte[p_src++]
        value |= byte[p_src++] << 8 
         
        term.str(string("Wait", CR))
        term.str(string("-- "))
        term.dec(value)
        term.tx(CR)
        pause(value)

      other:
        return


con

  { ------------- }
  {  B A S I C S  }
  { ------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1776                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]
  

dat  { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}